@@ -67,6 +67,10 @@ class DrawablePane(Extensible):
         'Width of the current sheet window, in single-width characters.'
         return self._scr.getmaxyx()[1] if self._scr else 80
 
+    @property
+    def currow(self):
+        return None
+
     def execCommand2(self, cmd, vdglobals=None):
         "Execute `cmd` with `vdglobals` as globals and this sheet's attributes as locals.  Return True if user cancelled."
 
@@ -112,9 +116,9 @@ class BaseSheet(DrawablePane):
 
     def __init__(self, *names, rows=UNLOADED, **kwargs):
         self._name = None   # initial cache value necessary for self.options
+        self._names = []
         self.loading = False
         self.names = list(names)
-        self.name = self.options.name_joiner.join(str(x) for x in self.names if x)
         self.source = None
         self.rows = rows      # list of opaque objects
         self._scr = None
@@ -194,7 +198,7 @@ class BaseSheet(DrawablePane):
 
         cmd = self.getCommand(longname or keystrokes)
         if not cmd:
-            vd.warning('no command for %s' % (longname or keystrokes))
+            vd.fail('no command for %s' % (longname or keystrokes))
             return False
 
         escaped = False
@@ -238,8 +242,10 @@ class BaseSheet(DrawablePane):
 
     @names.setter
     def names(self, names):
+        if self._names:
+            vd.addUndo(setattr, self, 'names', self._names)
         self._names = names
-        self.name = self.options.name_joiner.join(self.maybeClean(str(x)) for x in self._names)
+        self._name = self.options.name_joiner.join(self.maybeClean(str(x)) for x in self._names)
 
     @property
     def name(self):
@@ -250,8 +256,9 @@ class BaseSheet(DrawablePane):
     def name(self, name):
         'Set name without spaces.'
         if self._names:
-            vd.addUndo(setattr, self, '_names', self._names)
+            vd.addUndo(setattr, self, 'names', self._names)
         self._name = self.maybeClean(str(name))
+        self._names = [self._name]
 
     def maybeClean(self, s):
         'stub'
