@@ -145,7 +145,7 @@ class Plotter(BaseSheet):
         self.labels = []  # (x, y, text, attr, row)
         self.hiddenAttrs = set()
         self.needsRefresh = False
-        self.resetCanvasDimensions(self.windowHeight, self.windowWidth)
+        self.resetCanvasDimensions(1, 1)  #2171
 
     @property
     def nRows(self):
@@ -234,12 +234,28 @@ class Plotter(BaseSheet):
 
     def draw(self, scr):
         windowHeight, windowWidth = scr.getmaxyx()
-        disp_canvas_charset = self.options.disp_canvas_charset or ' o'
-        disp_canvas_charset += (256 - len(disp_canvas_charset)) * disp_canvas_charset[-1]
-
         if self.needsRefresh:
             self.render(windowHeight, windowWidth)
 
+        self.draw_pixels(scr)
+        self.draw_labels(scr)
+
+    def draw_empty(self, scr):
+        # use draw_empty() when calling draw_pixels() with clear_empty_squares=False
+        cursorBBox = self.plotterCursorBox
+        for char_y in range(0, self.plotheight//4):
+            for char_x in range(0, self.plotwidth//2):
+                cattr = ColorAttr()
+                ch = ' '
+                # draw cursor
+                if cursorBBox.contains(char_x*2, char_y*4) or \
+                    cursorBBox.contains(char_x*2+1, char_y*4+3):
+                    cattr = update_attr(cattr, colors.color_current_row)
+                scr.addstr(char_y, char_x, ch, cattr.attr)
+
+    def draw_pixels(self, scr, clear_empty_squares=True):
+        disp_canvas_charset = self.options.disp_canvas_charset or ' o'
+        disp_canvas_charset += (256 - len(disp_canvas_charset)) * disp_canvas_charset[-1]
         if self.pixels:
             cursorBBox = self.plotterCursorBox
             getPixelAttr = self.getPixelAttrRandom if self.options.disp_pixel_random else self.getPixelAttrMost
@@ -264,19 +280,26 @@ class Plotter(BaseSheet):
                             braille_num += pow2
                         pow2 *= 2
 
+                    ch = disp_canvas_charset[braille_num]
                     if braille_num != 0:
                         color = Counter(c for c in block_attrs if c).most_common(1)[0][0]
                         cattr = colors.get_color(color)
                     else:
                         cattr = ColorAttr()
+                        # don't erase empty squares, useful for subclasses that draw elements like reflines
+                        # before pixels are drawn
+                        if not clear_empty_squares:
+                            continue
 
+                    # draw cursor
                     if cursorBBox.contains(char_x*2, char_y*4) or \
                        cursorBBox.contains(char_x*2+1, char_y*4+3):
                         cattr = update_attr(cattr, colors.color_current_row)
 
                     if cattr.attr:
-                        scr.addstr(char_y, char_x, disp_canvas_charset[braille_num], cattr.attr)
+                        scr.addstr(char_y, char_x, ch, cattr.attr)
 
+    def draw_labels(self, scr):
         def _mark_overlap_text(labels, textobj):
             def _overlaps(a, b):
                 a_x1, _, a_txt, _, _ = a
@@ -318,7 +341,7 @@ class Plotter(BaseSheet):
                         cursorBBox = self.plotterCursorBox
                         for c in txt:
                             w = dispwidth(c)
-                            # check if the cursor contains the midpoint of the character box
+                            # draw cursor if the cursor contains the midpoint of the character cell
                             if cursorBBox.contains(char_x*2+1, char_y*4+2):
                                 char_attr = update_attr(cattr, colors.color_current_row)
                                 clipdraw(scr, char_y, char_x, c, char_attr, w)
@@ -393,6 +416,12 @@ class Canvas(Plotter):
                 if self.cursorBox.xmin == self.visibleBox.xmin and self.cursorBox.ymin == self.calcBottomCursorY():
                     realign_cursor = True
         super().resetCanvasDimensions(windowHeight, windowWidth)
+        # if window is not big enough to contain a particular margin, pretend that margin is 0
+        pvbox_x = pvbox_y = 0
+        if self.plotwidth > self.left_margin:
+            pvbox_x = self.left_margin
+        if self.plotheight > self.topMarginPixels:
+            pvbox_y = self.topMarginPixels
         if hasattr(self, 'legendwidth'):
             # +4 = 1 empty space after the graph + 2 characters for the legend prefixes of "1:", "2:", etc +
             #      1 character for the empty rightmost column
@@ -403,8 +432,10 @@ class Canvas(Plotter):
         else:
             pvbox_xmax = self.plotwidth-self.rightMarginPixels-1
         self.left_margin = min(self.left_margin, math.ceil(self.plotwidth * 1/3)//2*2)
-        self.plotviewBox = BoundingBox(self.left_margin, self.topMarginPixels,
-                                       pvbox_xmax, self.plotheight-self.bottomMarginPixels-1)
+        # enforce a minimum plotview box size of 1x1
+        pvbox_xmax = max(pvbox_xmax, 1)
+        pvbox_ymax = max(self.plotheight-self.bottomMarginPixels-1, 1)
+        self.plotviewBox = BoundingBox(pvbox_x, pvbox_y, pvbox_xmax, pvbox_ymax)
         if [self.plotheight, self.plotwidth] != old_plotsize:
             if hasattr(self, 'cursorBox') and self.cursorBox:
                 self.setCursorSizeInPlotterPixels(2, 4)
@@ -521,13 +552,14 @@ class Canvas(Plotter):
         'adjust visibleBox.xymin so that canvasPoint is plotted at plotterPoint'
         self.visibleBox.xmin = canvasPoint.x - self.canvasW(plotterPoint.x-self.plotviewBox.xmin)
         self.visibleBox.ymin = canvasPoint.y - self.canvasH(plotterPoint.y-self.plotviewBox.ymin)
-        self.refresh()
+        self.resetBounds()
 
     def zoomTo(self, bbox):
         'set visible area to bbox, maintaining aspectRatio if applicable'
         self.fixPoint(self.plotviewBox.xymin, bbox.xymin)
         self.xzoomlevel=bbox.w/self.canvasBox.w
         self.yzoomlevel=bbox.h/self.canvasBox.h
+        self.resetBounds()
 
     def incrZoom(self, incr):
         self.xzoomlevel *= incr
@@ -535,7 +567,7 @@ class Canvas(Plotter):
 
         self.resetBounds()
 
-    def resetBounds(self):
+    def resetBounds(self, refresh=True):
         'create canvasBox and cursorBox if necessary, and set visibleBox w/h according to zoomlevels.  then redisplay legends.'
         if not self.canvasBox:
             xmin, ymin, xmax, ymax = None, None, None, None
@@ -572,6 +604,8 @@ class Canvas(Plotter):
             self.cursorBox = Box(cb_xmin, cb_ymin, self.canvasCharWidth, self.canvasCharHeight)
 
         self.plotlegends()
+        if refresh:
+            self.refresh()
 
     def calcTopCursorY(self):
         'ymin for the cursor that will align its top with the top edge of the graph'
@@ -649,13 +683,13 @@ class Canvas(Plotter):
         else:
             return h
 
-    def scaleX(self, canvasX):
+    def scaleX(self, canvasX) -> int:
         'returns a plotter x coordinate'
-        return round(self.plotviewBox.xmin+(canvasX-self.visibleBox.xmin)*self.xScaler)
+        return self.plotviewBox.xmin+round((canvasX-self.visibleBox.xmin)*self.xScaler)
 
-    def scaleY(self, canvasY):
+    def scaleY(self, canvasY) -> int:
         'returns a plotter y coordinate'
-        return round(self.plotviewBox.ymin+(canvasY-self.visibleBox.ymin)*self.yScaler)
+        return self.plotviewBox.ymin+round((canvasY-self.visibleBox.ymin)*self.yScaler)
 
     def unscaleX(self, plotterX):
         'performs the inverse of scaleX, returns a canvas x coordinate'
@@ -683,6 +717,7 @@ class Canvas(Plotter):
         vd.cancelThread(*(t for t in self.currentThreads if t.name == 'plotAll_async'))
         self.labels.clear()
         self.resetCanvasDimensions(h, w)
+        self.resetBounds(refresh=False)
         self.render_async()
 
     @asyncthread
@@ -709,12 +744,12 @@ class Canvas(Plotter):
                 x1, y1 = float(x1), float(y1)
                 if xmin <= x1 <= xmax and ymin <= y1 <= ymax:
                     # equivalent to self.scaleX(x1) and self.scaleY(y1), inlined for speed
-                    x = plotxmin+(x1-xmin)*xfactor
+                    x = plotxmin+round((x1-xmin)*xfactor)
                     if invert_y:
-                        y = plotymax-(y1-ymin)*yfactor
+                        y = plotymax-round((y1-ymin)*yfactor)
                     else:
-                        y = plotymin+(y1-ymin)*yfactor
-                    self.plotpixel(round(x), round(y), attr, row)
+                        y = plotymin+round((y1-ymin)*yfactor)
+                    self.plotpixel(x, y, attr, row)
                 continue
 
             prev_x, prev_y = vertexes[0]
@@ -743,7 +778,6 @@ class Canvas(Plotter):
         self.source.deleteBy(lambda r,rows=rows: r in rows)
         self.reload()
 
-
 Plotter.addCommand('v', 'visibility', 'options.disp_graph_labels = not options.disp_graph_labels', 'toggle disp_graph_labels option')
 
 Canvas.addCommand(None, 'go-left', 'if cursorBox: sheet.cursorBox.xmin -= cursorBox.w', 'move cursor left by its width')
@@ -776,7 +810,7 @@ Canvas.addCommand('zz', 'zoom-cursor', 'zoomTo(cursorBox)', 'set visible bounds
 
 Canvas.addCommand('-', 'zoomout-cursor', 'tmp=cursorBox.center; incrZoom(options.disp_zoom_incr); fixPoint(plotviewBox.center, tmp)', 'zoom out from cursor center')
 Canvas.addCommand('+', 'zoomin-cursor', 'tmp=cursorBox.center; incrZoom(1.0/options.disp_zoom_incr); fixPoint(plotviewBox.center, tmp)', 'zoom into cursor center')
-Canvas.addCommand('_', 'zoom-all', 'sheet.canvasBox = None; sheet.visibleBox = None; sheet.xzoomlevel=sheet.yzoomlevel=1.0; resetBounds(); refresh()', 'zoom to fit full extent')
+Canvas.addCommand('_', 'zoom-all', 'sheet.canvasBox = None; sheet.visibleBox = None; sheet.xzoomlevel=sheet.yzoomlevel=1.0; resetBounds()', 'zoom to fit full extent')
 Canvas.addCommand('z_', 'set-aspect', 'sheet.aspectRatio = float(input("aspect ratio=", value=aspectRatio)); refresh()', 'set aspect ratio')
 
 # set cursor box with left click
