@@ -16,7 +16,8 @@ class TransposeSheet(Sheet):
         # rows become columns
         for row in Progress(self.source.rows, 'transposing'):
             self.addColumn(Column('_'.join(map(str, self.source.rowkey(row))),
-                                  getter=lambda c,origcol,row=row: origcol.getValue(row)))
+                                  getter=lambda c,origcol,row=row: origcol.getValue(row),
+                                  setter=lambda c,origcol,v,row=row: origcol.setValue(row, v)))
 
         # columns become rows
         self.rows = list(self.source.nonKeyVisibleCols)
