@@ -1,12 +1,32 @@
 import html
-from visidata import *
+import urllib.parse
+import copy
+import itertools
+import re
 
-option('html_title', '<h2>{sheet.name}</h2>', 'table header when saving to html')
+from visidata import VisiData, vd, Sheet, options, Column, Progress, IndexSheet, ItemColumn
 
-def open_html(p):
-    return HtmlTablesSheet(p.name, source=p)
+vd.option('html_title', '<h2>{sheet.name}</h2>', 'table header when saving to html')
+
+
+@VisiData.api
+def guess_html(vd, p):
+    with p.open() as fp:
+        r = fp.read(10240)
+        if r.strip().startswith('<'):
+            m = re.search(r, r'charset=(\S+)')
+            if m:
+                encoding = m.group(0)
+            else:
+                encoding = None
+            return dict(filetype='html', _likelihood=1, encoding=encoding)
+
+@VisiData.api
+def open_html(vd, p):
+    return HtmlTablesSheet(p.base_stem, source=p)
+
+VisiData.open_htm = VisiData.open_html
 
-open_htm = open_html
 
 class HtmlTablesSheet(IndexSheet):
     rowtype = 'sheets'  # rowdef: HtmlTableSheet (sheet.html = lxml.html.HtmlElement)
@@ -14,21 +34,27 @@ class HtmlTablesSheet(IndexSheet):
         Column('tag', width=0, getter=lambda col,row: row.html.tag),
         Column('id', getter=lambda col,row: row.html.attrib.get('id')),
         Column('classes', getter=lambda col,row: row.html.attrib.get('class')),
+        Column('title', getter=lambda col,row: row.html.attrib.get('title')),
+        Column('aria_label', getter=lambda col,row: row.html.attrib.get('aria-label')),
+        Column('caption', getter=lambda col,row: row.html.xpath('normalize-space(./caption)') if row.html.xpath('./caption') else None, cache=True),
+        Column('summary', getter=lambda col,row: row.html.attrib.get('summary')),
+        Column('heading', getter=lambda col,row: row.html.xpath('normalize-space(./preceding-sibling::*[self::h1 or self::h2 or self::h3 or self::h4 or self::h5 or self::h6][1])') or None, cache=True),
     ]
     def iterload(self):
-        import lxml.html
-        from lxml import etree
-        utf8_parser = etree.HTMLParser(encoding='utf-8')
-        with self.source.open_text() as fp:
-            html = lxml.html.etree.parse(fp, parser=utf8_parser)
+        lxml = vd.importExternal('lxml')
+        from lxml import html
+        with self.source.open(encoding='utf-8') as fp:
+            doc = html.parse(fp, parser=vd.utf8_parser, base_url=self.source.given)
         self.setKeys([self.column('name')])
         self.column('keys').hide()
         self.column('source').hide()
 
-        for i, e in enumerate(html.iter('table')):
+        for i, e in enumerate(doc.iter('table')):
             if e.tag == 'table':
                 yield HtmlTableSheet(e.attrib.get("id", "table_" + str(i)), source=e, html=e)
 
+        yield HtmlLinksSheet(*self.names, 'links', source=doc, html=doc.getroot())
+
 
 def is_header(elem):
     scope = elem.attrib.get('scope', '')
@@ -39,14 +65,55 @@ def is_header(elem):
 
     return False
 
+class HtmlLinksSheet(Sheet):
+    rowtype = 'links'  #  rowdef: tuple(element, attribute, link, pos)
+    columns = [
+        ItemColumn('element', 0, width=0),
+        Column('tag', getter=lambda c,r:r[0].tag),
+        ItemColumn('attribute', 1),
+        Column('text', getter=lambda c,r:r[0].text),
+        ItemColumn('link', 2, width=40),
+    ]
+    def iterload(self):
+        lxml = vd.importExternal('lxml')
+        from lxml.html import iterlinks
+        root = self.source.getroot()
+        root.make_links_absolute(self.source.docinfo.URL, handle_failures='ignore')
+        yield from iterlinks(root)
+
+    def openRow(self, row):
+        return vd.openSource(row[2])
+
+class HtmlElementsSheet(Sheet):
+    rowtype = 'links'  #  dict
+    columns = [
+        ItemColumn('__element__', width=0),
+    ]
+    def iterload(self):
+        yield from self.source.iterlinks()
+        return
+        for r in self.source.iter(self.source_tag):
+            row = copy.copy(r.attrib)
+            row['__element__'] = r
+            if row.get('href'):
+                row['href'] = urllib.parse.urljoin(self.source.URL, row.get('href'))
+
+            yield row
+
+    def openRow(self, row):
+        return vd.openSource(row.url)
+
+
 class HtmlTableSheet(Sheet):
     rowtype = 'rows'  #  list of strings
     columns = []
 
     def iterload(self):
+        import lxml
         headers = []
 
         maxlinks = {}  # [colnum] -> nlinks:int
+        ncols = 0
 
         for rownum, r in enumerate(self.source.iter('tr')):
             row = []
@@ -62,8 +129,14 @@ class HtmlTableSheet(Sheet):
             for cell in r.getchildren():
                 colspan = int(cell.attrib.get('colspan', 1))
                 rowspan = int(cell.attrib.get('rowspan', 1))
+                if isinstance(cell, lxml.etree.CommentBase):
+                    continue
                 cellval = ' '.join(x.strip() for x in cell.itertext())  # text only without markup
-                links = [x.get('href') for x in cell.iter('a')]
+                links = [
+                    vd.callNoExceptions(urllib.parse.urljoin, self.source.base_url, x.get('href')) or x.get('href')
+                        for x in cell.iter('a')
+                ]
+
                 maxlinks[colnum] = max(maxlinks.get(colnum, 0), len(links))
 
                 if is_header(cell):
@@ -78,20 +151,24 @@ class HtmlTableSheet(Sheet):
                         cellval = ''   # use empty non-None value for subsequent rows in the rowspan
                 else:
                     while colnum >= len(row):
-                        row.append(None)
+                        row.append((None, []))
                     row[colnum] = (cellval, links)
 
                 colnum += colspan
 
             if any(row):
                 yield row
+                ncols = max(ncols, colnum)
 
         self.columns = []
         if headers:
             it = itertools.zip_longest(*headers, fillvalue='')
         else:
-            it = [list(x) for x in self.rows[0]]
-            self.rows = self.rows[1:]
+            if len(self.rows) > 0:
+                it = list(list(x) for x in self.rows.pop(0))
+                it += [''] * (ncols-len(it))
+            else:
+                it = []
 
         for colnum, names in enumerate(it):
             name = '_'.join(str(x) for x in names if x)
@@ -130,7 +207,20 @@ def save_html(vd, p, *vsheets):
                     fp.write('</tr>\n')
 
             fp.write('</table>')
-            vd.status('%s save finished' % p)
+
+
+@VisiData.lazy_property
+def utf8_parser(vd):
+    lxml = vd.importExternal('lxml')
+    return lxml.html.HTMLParser(encoding='utf-8')
+#    return lxml.etree.HTMLParser(encoding='utf-8')
+
+
+@VisiData.api
+def HTML(vd, s):
+    lxml = vd.importExternal('lxml')
+    from lxml import html
+    return html.fromstring(s, parser=vd.utf8_parser)
 
 
 VisiData.save_htm = VisiData.save_html
