@@ -29,10 +29,9 @@ Some new commands on this sheet operate on all selected columns on the source sh
 - {help.commands.type_int_selected}
 - or `g` with any standard type to set type of selected source columns to that type
 
-## As usual
+Other commands (not specific to Columns Sheet):
 
 - {help.commands.setcol_input}
-{sheet.help_columns}
 '''
 
     class ValueColumn(Column):
@@ -47,15 +46,15 @@ Some new commands on this sheet operate on all selected columns on the source sh
             ColumnAttr('name', help='rename the column on the source sheet'),
             ColumnAttr('keycol', type=int, width=0),
             ColumnAttr('width', type=int, help='set the column width (`0` to hide completely)'),
-            ColumnAttr('height', type=int, max_help=1, help='set a maximum height for the row, if this column will fill it'),
+            ColumnAttr('height', type=int, disp_expert=1, help='set a maximum height for the row, if this column will fill it'),
             ColumnAttr('hoffset', type=int, width=0),
             ColumnAttr('voffset', type=int, width=0),
             ColumnAttr('type', 'typestr', help='convert all values to a specific type'),
             ColumnAttr('fmtstr', help='use a custom format string, either C-style (`%0.4f`) or Python-style (`{{:0.4f}}`)'),
-            ColumnAttr('formatter', max_help=1, help='use a custom format function (**{col.help_formatters}**)'),
-            ColumnAttr('displayer', max_help=1, help='use a custom display function (**{col.help_displayers}**)'),
+            ColumnAttr('formatter', disp_expert=1, help='use a custom format function (**{col.help_formatters}**)'),
+            ColumnAttr('displayer', disp_expert=1, help='use a custom display function (**{col.help_displayers}**)'),
             ValueColumn('value', help='change the value of this cell on the source sheet'),
-            ColumnAttr('expr', max_help=1, help='change the main column parameter'),
+            ColumnAttr('expr', disp_expert=1, help='change the main column parameter'),
             ColumnAttr('ncalcs', type=int, width=0, cache=False),
             ColumnAttr('maxtime', type=float, width=0, cache=False),
             ColumnAttr('totaltime', type=float, width=0, cache=False),
