@@ -84,7 +84,7 @@ class SettingsMgr(collections.OrderedDict):
         if d:
             for m in self._mappings(obj or vd.activeSheet):
                 v = d.get(self.objname(m))
-                if v:
+                if v is not None:
                     return v
 
     def iter(self, obj=None):
@@ -107,16 +107,17 @@ class SettingsMgr(collections.OrderedDict):
 
 
 class Command:
-    def __init__(self, longname, execstr, helpstr='', module='', deprecated=False):
+    def __init__(self, longname, execstr, helpstr='', module='', replay=True, deprecated=False):
         self.longname = longname
         self.execstr = execstr
         self.helpstr = helpstr
         self.module = module
         self.deprecated = deprecated
+        self.replayable = replay
 
 
 class Option:
-    def __init__(self, name, value, description='', module='', help='', max_help=10):
+    def __init__(self, name, value, description='', module='', help=''):
         # description gets shows on the manpage and the optionssheet; help is shown on the sidebar while editing
         self.name = name
         self.value = value
@@ -125,7 +126,6 @@ class Option:
         self.replayable = False
         self.sheettype = BaseSheet
         self.module = module
-        self.max_help = max_help
 
     def __str__(self):
         return str(self.value)
@@ -299,7 +299,7 @@ def _resolve_optalias(vd, optname, optval):
 
 
 @VisiData.api
-def option(vd, name, default, description, replay=False, sheettype=BaseSheet, help:str='', max_help=10):
+def option(vd, name, default, description, replay=False, sheettype=BaseSheet, help:str=''):
     '''Declare a new option.
 
    - `name`: name of option
@@ -312,7 +312,6 @@ def option(vd, name, default, description, replay=False, sheettype=BaseSheet, he
     opt.replayable = replay
     opt.sheettype=sheettype
     opt.extrahelp = help
-    opt.max_help = max_help
     return opt
 
 
@@ -320,12 +319,12 @@ def option(vd, name, default, description, replay=False, sheettype=BaseSheet, he
 def theme_option(vd, name, *args, **kwargs):
     if name.startswith('color_'):
         kwargs.setdefault('help', vd.help_color)
-    return vd.option(name, *args, **kwargs, max_help=-1)
+    return vd.option(name, *args, **kwargs)
 
 
 @BaseSheet.class_api
 @classmethod
-def addCommand(cls, keystrokes, longname, execstr, helpstr='', **kwargs):
+def addCommand(cls, keystrokes, longname, execstr, helpstr='', replay=True, **kwargs):
     '''Add a new command to *cls* sheet type.
 
     - *keystrokes*: default keybinding, including **prefixes**.
@@ -333,9 +332,10 @@ def addCommand(cls, keystrokes, longname, execstr, helpstr='', **kwargs):
     - *execstr*: Python statement to pass to `exec()`'ed when the command is executed.
     - *helpstr*: help string shown in the **Commands Sheet**.
     '''
-    vd.commands.set(longname, Command(longname, execstr, helpstr=helpstr, module=vd.importingModule, **kwargs), cls)
+    cmd = Command(longname, execstr, helpstr=helpstr, module=vd.importingModule, replay=replay, **kwargs)
+    vd.commands.set(longname, cmd, cls)
     if keystrokes:
-        vd.bindkeys.set(vd.prettykeys(keystrokes.replace(' ', '')), longname, cls)
+        vd.bindkey(keystrokes, longname, cls)
     return longname
 
 def _command(cls, binding, longname, helpstr, **kwargs):
@@ -346,12 +346,19 @@ def _command(cls, binding, longname, helpstr, **kwargs):
     return decorator
 
 BaseSheet.command = classmethod(_command)
-globalCommand = BaseSheet.addCommand
+globalCommand = BaseSheet.addCommand   # to be deprecated
+
 
 @VisiData.api
-def bindkey(vd, keystrokes, longname):
+def bindkey(vd, keystrokes, longname, obj='BaseSheet'):
     'Bind *keystrokes* to *longname* on BaseSheet and unbind more-specific bindings of keystrokes.'
-    vd.bindkeys[vd.prettykeys(keystrokes)] = {'BaseSheet': longname}
+    vd.bindkeys.set(vd.prettykeys(keystrokes.replace(' ', '')), longname, obj)
+
+@VisiData.api
+def unbindkey(vd, keystrokes, obj='BaseSheet'):
+    'Bind *keystrokes* to *longname* on BaseSheet and unbind more-specific bindings of keystrokes.'
+    vd.bindkeys.unset(vd.prettykeys(keystrokes.replace(' ', '')), obj)
+
 
 @BaseSheet.class_api
 @classmethod
@@ -360,14 +367,14 @@ def bindkey(cls, keystrokes, longname):
     oldlongname = vd.bindkeys._get(keystrokes, cls)
     if oldlongname:
         vd.warning('%s was already bound to %s' % (keystrokes, oldlongname))
-    vd.bindkeys.set(vd.prettykeys(keystrokes), longname, cls)
+    vd.bindkey(keystrokes, longname, cls)
 
 @BaseSheet.class_api
 @classmethod
 def unbindkey(cls, keystrokes):
     '''Unbind `keystrokes` on a `<SheetType>`.
     May be necessary to avoid a warning when overriding a binding on the same exact class.'''
-    vd.bindkeys.unset(vd.prettykeys(keystrokes), cls)
+    vd.unbindkey(keystrokes, cls)
 
 @BaseSheet.api
 def getCommand(sheet, cmd):
@@ -376,29 +383,32 @@ def getCommand(sheet, cmd):
         return cmd
 
     longname = cmd
-    while vd.bindkeys._get(longname, obj=sheet):
+    while vd.bindkeys._get(longname, obj=sheet) is not None:
         longname = vd.bindkeys._get(longname, obj=sheet)
 
     return vd.commands._get(longname, obj=sheet)
 
 
 @VisiData.api
-def loadConfigFile(vd, fn='', _globals=None):
+def loadConfigFile(vd, fn=''):
     p = visidata.Path(fn or vd.options.config)
-    if _globals is None:
-        _globals = vd.getGlobals()
     if p.exists():
+        newdefs = {}
         try:
             with open(p) as fd:
                 code = compile(fd.read(), str(p), 'exec')
             vd.importingModule = 'visidatarc'
-            exec(code, _globals)
+            exec(code, vd.getGlobals(), newdefs)
         except Exception as e:
             vd.exceptionCaught(e)
         finally:
             vd.importingModule = None
 
-    vd.addGlobals(_globals)
+        keys = newdefs.get('__all__', None)
+        if keys:
+            vd.addGlobals({k:newdefs[k] for k in keys})
+        else:
+            vd.addGlobals(newdefs)
 
 
 def addOptions(parser):
@@ -470,17 +480,21 @@ def loadConfigAndPlugins(vd, args=AttrDict()):
 
     # user customisations in config file in standard location
     if vd.options.config:
-        vd.loadConfigFile(vd.options.config, vd.getGlobals())
+        vd.loadConfigFile(vd.options.config)
 
 
 @VisiData.api
-def importModule(vd, pkgname):
+def importModule(vd, pkgname, symbols=[]):
     'Import the given *pkgname*, setting vd.importingModule to *pkgname* before import and resetting to None after.'
     modparts = pkgname.split('.')
     vd.importingModule = modparts[-1]
     r = importlib.import_module(pkgname)
     vd.importingModule = None
     vd.importedModules.append(r)
+    vd.addGlobals({pkgname:r})
+    if symbols:
+        vd.addGlobals({k:getattr(r, k) for k in symbols if hasattr(r, k)})
+
     return r
 
 
@@ -488,24 +502,12 @@ def importModule(vd, pkgname):
 def importSubmodules(vd, pkgname):
     'Import all files below the given *pkgname*'
     import pkgutil
-    import os.path
 
     m = vd.importModule(pkgname)
     for module in pkgutil.walk_packages(m.__path__):
         vd.importModule(pkgname + '.' + module.name)
 
 
-@VisiData.api
-def importStar(vd, pkgname):
-    'Add all symbols from *pkgname* into visidata globals.'
-    import pkgutil
-    import os.path
-
-    m = vd.importModule(pkgname)
-    vd.addGlobals({pkgname:m})
-    vd.addGlobals(m.__dict__)
-
-
 @VisiData.api
 def importExternal(vd, modname, pipmodname=''):
     pipmodname = pipmodname or modname
