@@ -1,8 +1,6 @@
 import os
-import io
-import sys
+import shutil
 import stat
-import locale
 import subprocess
 import contextlib
 try:
@@ -11,19 +9,26 @@ try:
 except ImportError:
     pass # pwd,grp modules not available on Windows
 
-from visidata import Column, Sheet, LazyComputeRow, asynccache, options, option, globalCommand
-from visidata import Path, ENTER, date, asyncthread, confirm, fail, FileExistsError, VisiData
-from visidata import CellColorizer, RowColorizer, modtime, filesize, vstat
+from visidata import Column, Sheet, LazyComputeRow, asynccache, BaseSheet, vd
+from visidata import Path, ENTER, asyncthread, VisiData
+from visidata import modtime, filesize, vstat, Progress, TextSheet
+from visidata.type_date import date
 
 
-option('dir_recurse', False, 'walk source path recursively on DirSheet')
-option('dir_hidden', False, 'load hidden files on DirSheet')
+vd.option('dir_depth', 0, 'folder recursion depth on DirSheet')
+vd.option('dir_hidden', False, 'load hidden files on DirSheet')
+
+
+@VisiData.api
+def guess_dir(vd, p):
+    if p.is_dir():
+        return dict(filetype='dir')
 
 
 @VisiData.lazy_property
 def currentDirSheet(p):
     'Support opening the current DirSheet from the vdmenu'
-    return DirSheet('.', source=Path('.'))
+    return DirSheet(Path('.').absolute().name, source=Path('.'))
 
 @asyncthread
 def exec_shell(*args):
@@ -34,38 +39,48 @@ def exec_shell(*args):
         vd.push(TextSheet(' '.join(args), source=lines))
 
 
-def open_dir(p):
-    return DirSheet(p.name, source=p)
+@VisiData.api
+def open_dir(vd, p):
+    if p.is_dir():
+        return DirSheet(p.base_stem, source=p)
+    if p.is_file():
+        vd.status(f'opening {p.given} as txt')
+        return vd.open_txt(p)
+    vd.warning(f'could not determine file type for {p.given}')
+    return DirSheet(p.base_stem, source=p)
 
-def open_fdir(p):
-    return FileListSheet(p.name, source=p)
+@VisiData.api
+def open_fdir(vd, p):
+    return FileListSheet(p.base_stem, source=p)
 
-def addShellColumns(cmd, sheet):
-    shellcol = ColumnShell(cmd, source=sheet, width=0)
+@VisiData.api
+def addShellColumns(vd, cmd, sheet, curcol=None):
+    shellcol = ColumnShell(cmd, source=sheet, width=0, curcol=curcol)
     sheet.addColumnAtCursor(
-            shellcol,
-            Column(cmd+'_stdout', srccol=shellcol, getter=lambda col,row: col.srccol.getValue(row)[0]),
-            Column(cmd+'_stderr', srccol=shellcol, getter=lambda col,row: col.srccol.getValue(row)[1]))
+            Column(cmd+'_stdout', type=bytes.rstrip, srccol=shellcol, getter=lambda col,row: col.srccol.getValue(row)[0]),
+            Column(cmd+'_stderr', type=bytes.rstrip, srccol=shellcol, getter=lambda col,row: col.srccol.getValue(row)[1]),
+            shellcol)
 
 
 class ColumnShell(Column):
-    def __init__(self, name, cmd=None, **kwargs):
+    def __init__(self, name, cmd=None, curcol=None, **kwargs):
         super().__init__(name, **kwargs)
         self.expr = cmd or name
+        self.curcol = curcol
 
     @asynccache(lambda col,row: (col, col.sheet.rowid(row)))
     def calcValue(self, row):
         try:
             import shlex
             args = []
-            context = LazyComputeRow(self.source, row)
+            context = LazyComputeRow(self.source, row, curcol=self.curcol)
             for arg in shlex.split(self.expr):
                 if arg.startswith('$'):
-                    args.append(str(context[arg[1:]]))
-                else:
-                    args.append(arg)
+                    arg = shlex.quote(str(context[arg[1:]]))
+                args.append(arg)
 
-            p = subprocess.Popen(args, stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
+            p = subprocess.Popen([os.getenv('SHELL', 'bash'), '-c', shlex.join(args)],
+                    stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
             return p.communicate()
         except Exception as e:
             vd.exceptionCaught(e)
@@ -73,14 +88,28 @@ class ColumnShell(Column):
 
 class DirSheet(Sheet):
     'Sheet displaying directory, using ENTER to open a particular file.  Edited fields are applied to the filesystem.'
+    guide = '''
+        # Directory Sheet
+        This is a list of files in the {sheet.displaySource} folder.
+
+        - {help.commands.open_row_file}
+        - {help.commands.open_rows}
+        - {help.commands.open_dir_parent}
+        - {help.commands.sysopen_row}
+
+        ## Options (must reload to take effect)
+
+        - {help.options.dir_depth}
+        - {help.options.dir_hidden}
+    '''
     rowtype = 'files' # rowdef: Path
     defer = True
     columns = [
         Column('directory',
-            getter=lambda col,row: str(row.parent) if str(row.parent) == '.' else str(row.parent) + '/',
+            getter=lambda col,row: str(row.parent) if str(row.parent) in ('.', '/') else str(row.parent) + '/',
             setter=lambda col,row,val: col.sheet.moveFile(row, val)),
         Column('filename',
-            getter=lambda col,row: row.name + row.suffix,
+            getter=lambda col,row: row._path.name,
             setter=lambda col,row,val: col.sheet.renameFile(row, val)),
         Column('abspath', width=0, type=str,
             getter=lambda col,row: row,
@@ -142,40 +171,47 @@ class DirSheet(Sheet):
 
     def removeFile(self, path):
         if path.is_dir():
-            os.rmdir(path)
+            if self.options.safety_first:
+                os.rmdir(path)
+            else:
+                shutil.rmtree(path)  #1965
         else:
             path.unlink()
 
-    def deleteSourceRow(self, r):
+    def commitDeleteRow(self, r):
         self.removeFile(r)
 
+    def newRow(self):
+        vd.fail('new file not supported')
+
     def iterload(self):
-        def _walkfiles(p):
+        hidden_files = self.options.dir_hidden
+
+        def _walkfiles(p, dir_depth:int=0):
             basepath = str(p)
             for folder, subdirs, files in os.walk(basepath):
                 subfolder = folder[len(basepath)+1:]
+                if not hidden_files and subfolder.startswith('.'): continue
                 if subfolder in ['.', '..']: continue
 
                 fpath = Path(folder)
-                yield fpath
+                if str(fpath) != str(p):
+                    yield fpath
 
                 for fn in files:
                     yield fpath/fn
 
-        def _listfiles(p):
-            basepath = str(p)
-            for fn in os.listdir(basepath):
-                yield p/fn
-
+                if dir_depth < len(fpath.parents)-len(p.parents)+1:
+                    for d in subdirs:
+                        yield fpath/d
+                    subdirs.clear()
 
         basepath = str(self.source)
 
         folders = set()
-        f = _walkfiles if self.options.dir_recurse else _listfiles
 
-        hidden_files = options.dir_hidden
-        for p in f(self.source):
-            if hidden_files and p.name.startswith('.'):
+        for p in _walkfiles(self.source, self.options.dir_depth):
+            if not hidden_files and str(p).startswith('.') and not str(p).startswith('..'):
                 continue
 
             yield p
@@ -196,11 +232,14 @@ class DirSheet(Sheet):
         self._deferredDels.clear()
         self.reload()
 
+    def getDefaultSaveName(sheet):
+        return sheet.name + '.' + sheet.options.save_filetype
+
 
 class FileListSheet(DirSheet):
     _ordering = []
     def iterload(self):
-        for fn in self.source.open_text():
+        for fn in self.source.open():
             yield Path(fn.rstrip())
 
 
@@ -211,11 +250,45 @@ def inputShell(vd):
         vd.warning('no $column in command')
     return cmd
 
-globalCommand('', 'open-dir-current', 'vd.push(vd.currentDirSheet)', 'open Directory Sheet: browse properties of files in current directory')
+DirSheet.addCommand('`', 'open-dir-parent', 'vd.push(openSource(source.parent if source.resolve()!=Path(".").resolve() else os.path.dirname(source.resolve())))', 'open parent directory')  #1801
+BaseSheet.addCommand('', 'open-dir-current', 'vd.push(vd.currentDirSheet)', 'open Directory Sheet: browse properties of files in current directory')
 
-Sheet.addCommand('z;', 'addcol-sh', 'cmd=inputShell(); addShellColumns(cmd, sheet)', 'create new column from bash expression, with $columnNames as variables')
+Sheet.addCommand('z;', 'addcol-shell', 'cmd=inputShell(); addShellColumns(cmd, sheet, curcol=cursorCol)', 'create new column from bash expression, with $columnNames as variables')
 
-DirSheet.addCommand(ENTER, 'open-row', 'vd.push(openSource(cursorRow or fail("no row"), filetype="dir" if cursorRow.is_dir() else LazyComputeRow(sheet, cursorRow).ext))', 'open current file as a new sheet')
+DirSheet.addCommand(ENTER, 'open-row-file', 'vd.push(openSource(cursorRow or fail("no row"), filetype="dir" if cursorRow.is_dir() else LazyComputeRow(sheet, cursorRow).ext))', 'open current file as a new sheet')
 DirSheet.addCommand('g'+ENTER, 'open-rows', 'for r in selectedRows: vd.push(openSource(r))', 'open selected files as new sheets')
 DirSheet.addCommand('^O', 'sysopen-row', 'launchEditor(cursorRow)', 'open current file in external $EDITOR')
 DirSheet.addCommand('g^O', 'sysopen-rows', 'launchEditor(*selectedRows)', 'open selected files in external $EDITOR')
+
+DirSheet.addCommand('y', 'copy-row', 'copy_files([cursorRow], inputPath("copy to dest: "))', 'copy file to given directory *path*')
+DirSheet.addCommand('gy', 'copy-selected', 'copy_files(selectedRows, inputPath("copy to dest: ", value=cursorRow.given))', 'copy selected files to given directory *path*')
+
+DirSheet.addCommand('z'+ENTER, 'open-row-filetype', 'ft = input("filetype: ", type="filetype", value=options.filetype or LazyComputeRow(sheet, cursorRow).ext); vd.push(openSource(cursorRow, filetype=ft) or fail(f"file {cursorDisplay} does not exist"))', 'open file in current row as input filetype')
+
+
+@DirSheet.api
+@asyncthread
+def copy_files(sheet, paths, dest):
+    destdir = Path(dest)
+    destdir.is_dir() or vd.fail('target must be directory')
+    vd.status('copying %s %s to %s' % (len(paths), sheet.rowtype, destdir))
+    os.makedirs(destdir, exist_ok=True)
+    for srcpath in Progress(paths, gerund='copying'):
+        try:
+            destpath = destdir/str(srcpath._path.name)
+            if srcpath.is_dir():
+                shutil.copytree(srcpath, destpath)
+            else:
+                shutil.copyfile(srcpath, destpath)
+        except Exception as e:
+            vd.exceptionCaught(e)
+
+
+vd.addGlobals({
+    'DirSheet': DirSheet
+})
+
+vd.addMenuItems('''
+    Column > Add column > shell > addcol-shell
+    Open > file in row > open-row-filetype
+''')
